# Generated by roxygen2: do not edit by hand

S3method(diagnostics,cvasi_simulate)
S3method(diagnostics,default)
S3method(num_info,cvasi_fit)
S3method(num_info,cvasi_simulate)
S3method(num_info,default)
S3method(plot,cvasi_drc)
S3method(plot,cvasi_simulate)
S3method(plot,lik_profile)
S3method(plot,param_space)
export("%>%")
export(Algae_Simple)
export(Algae_TKTD)
export(Algae_Weber)
export(CalibrationSet)
export(DEB_Daphnia)
export(DEB_abj)
export(DEBtox)
export(ExposureSeries)
export(GUTS_RED_IT)
export(GUTS_RED_SD)
export(GUTS_SD)
export(Lemna_SETAC)
export(Lemna_Schmitt)
export(Lemna_SchmittThold)
export(Magma)
export(Myrio)
export(Myrio_log)
export(SamX)
export(batch)
export(cache_controls)
export(calibrate)
export(caliset)
export(ci_from_hessian)
export(diagnostics)
export(dose_response)
export(effect)
export(epx)
export(epx_mtw)
export(explore_space)
export(fit_growth)
export(fit_tktd)
export(fx)
export(get_model)
export(get_model_name)
export(get_param)
export(get_tag)
export(get_times)
export(import_morse)
export(import_swash)
export(import_toxswa)
export(is_DEB)
export(is_GUTS)
export(is_GUTS_IT)
export(is_GUTS_SD)
export(is_Lemna)
export(is_LemnaThreshold)
export(is_deb)
export(is_guts)
export(is_guts_it)
export(is_guts_sd)
export(is_lemna)
export(is_scenario)
export(lik_profile)
export(log_disable)
export(log_enable)
export(log_envir)
export(log_lik)
export(log_msg)
export(log_scenarios)
export(morse)
export(no_exposure)
export(num_info)
export(parameter_set)
export(pll_debug)
export(plot_epx)
export(plot_lik_profile)
export(plot_param_space)
export(plot_ppc)
export(plot_scenario)
export(plot_sd)
export(pull_metadata)
export(sequence)
export(set_bounds)
export(set_endpoints)
export(set_exposure)
export(set_forcings)
export(set_init)
export(set_moa)
export(set_mode_of_action)
export(set_noexposure)
export(set_notransfer)
export(set_nowindow)
export(set_param)
export(set_tag)
export(set_times)
export(set_transfer)
export(set_window)
export(simulate)
export(simulate_batch)
export(solver)
export(survival)
export(td2cs)
export(tox_data)
exportClasses(Algae)
exportClasses(AlgaeSimple)
exportClasses(AlgaeSimpleScenario)
exportClasses(AlgaeTKTD)
exportClasses(AlgaeTKTDScenario)
exportClasses(AlgaeWeber)
exportClasses(AlgaeWeberScenario)
exportClasses(CalibrationSet)
exportClasses(Deb)
exportClasses(DebAbj)
exportClasses(DebDaphnia)
exportClasses(DebTox)
exportClasses(EffectScenario)
exportClasses(ExposureSeries)
exportClasses(Guts)
exportClasses(GutsIt)
exportClasses(GutsRedIt)
exportClasses(GutsRedSd)
exportClasses(GutsSd)
exportClasses(Lemna)
exportClasses(LemnaSchmitt)
exportClasses(LemnaSchmittScenario)
exportClasses(LemnaSetac)
exportClasses(LemnaSetacScenario)
exportClasses(Magma)
exportClasses(NoExposureSeries)
exportClasses(ParameterSet)
exportClasses(ScenarioSequence)
exportClasses(Transferable)
exportClasses(parameter_set)
exportMethods("[")
exportMethods("[[")
exportMethods("[[<-")
exportMethods(effect)
exportMethods(get_model)
exportMethods(get_param)
exportMethods(get_tag)
exportMethods(get_times)
exportMethods(length)
exportMethods(set_bounds)
exportMethods(set_param)
exportMethods(set_times)
importFrom(deSolve,diagnostics)
importFrom(deSolve,diagnostics.deSolve)
importFrom(deSolve,diagnostics.default)
importFrom(deSolve,ode)
importFrom(lifecycle,deprecated)
importFrom(lifecycle,is_present)
importFrom(magrittr,"%>%")
importFrom(methods,"slot<-")
importFrom(methods,.hasSlot)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,selectMethod)
importFrom(methods,show)
importFrom(methods,slot)
importFrom(methods,slotNames)
importFrom(rlang,abort)
importFrom(rlang,enquos)
importFrom(rlang,eval_tidy)
importFrom(rlang,inform)
importFrom(rlang,warn)
importFrom(stats,optim)
importFrom(stats,setNames)
importFrom(utils,capture.output)
importFrom(utils,head)
importFrom(utils,installed.packages)
importFrom(utils,sessionInfo)
importFrom(utils,tail)
useDynLib(cvasi, .registration = TRUE)
