This folder contains exposure time series of the fungicide propiconazole which
were predicted using FOCUS surface water models. The data has been published
as part of the supporting information (Appendix E) by EFSA (2018).

Original filenames:
Propiconazole apple simulations.xlsx
Propiconazole spring cereals simulations.xlsx

Literature:
EFSA Scientific Opinion on TKTD models, 2018. doi:10.2903/j.efsa.2018.5377
