#' Macrophyte models
#'
#' Population models of standard test macrophytes, such as *Lemna spp.*
#'
#' Available macrophyte models:
#' - [Lemna][Lemna-models]
#' - [Magma][Magma()]
#'
#' @inheritSection Transferable Biomass transfer
#'
#' @name Macrophyte-models
#' @family macrophyte models
#' @family models
#' @seealso [Scenarios]
NULL
