# Generated by roxyglobals: do not edit by hand

utils::globalVariables(c(
  "obs", # <calibrate_list>
  "pred", # <calibrate_list>
  "log_obs", # <calibrate_list>
  "log_pred", # <calibrate_list>
  "endpoint", # <dose_response>
  "window.start", # <effect_scenario>
  "window.end", # <effect_scenario>
  "scenario", # <effect_scenario>
  "endpoint", # <epx_min_win>
  "level", # <epx_min_win>
  "EPx", # <epx_min_win>
  "window.start", # <epx_min_win>
  "size", # <epx_min_win>
  "ep", # <epx_binary_search>
  "level", # <epx_binary_search>
  "mf", # <epx_binary_search>
  "tags", # <explore_space>
  "LLR_accept", # <explore_space>
  "time", # <fit_growth_lemna>
  "time", # <fit_growth_magma>
  "time", # <fit_tktd_lemna_setac>
  "time", # <fit_tktd_magma>
  "V1", # <import_toxswa>
  "V3", # <import_toxswa>
  "value", # <import_toxswa>
  "name", # <import_toxswa>
  "time", # <import_toxswa>
  "timestamp", # <import_toxswa>
  "tags", # <profile_par>
  ".", # <format_table>
  "par_value", # <plot.lik_profile>
  "log_lik_rat", # <plot.lik_profile>
  "start", # <plot.lik_profile>
  "LL", # <plot.param_space>
  "LLR_quality", # <plot.param_space>
  "trial", # <plot_sd>
  "time", # <plot_ppc>
  "trial", # <plot_ppc>
  "obs", # <plot_ppc_combi>
  "PPC", # <plot_ppc_combi>
  "pred", # <plot_ppc_combi>
  "study", # <plot_ppc_combi>
  "window.start", # <plot_epx>
  "window.end", # <plot_epx>
  "endpoint", # <plot_epx>
  "len", # <plot_epx>
  "EPx", # <plot_epx>
  ".data", # <plot_epx>
  "scenario", # <pull_metadata>
  "context", # <pull_metadata>
  "tag", # <set_param_multi>
  "model", # <set_param_multi>
  "name", # <set_param_multi>
  "desc", # <set_param_multi>
  "S", # <survival>
  "time", # <surv_data>
  "Nsurv", # <surv_data>
  "conc", # <surv_data>
  NULL
))
