# scaled Ci, M=Ci

    WAoAAAACAAQDAAACAwAAAAMTAAAABQAAAA4AAAApAAAAAAAAAAA/uZmZmZmZmj/JmZmZmZma
    P9MzMzMzMzQ/2ZmZmZmZmj/gAAAAAAAAP+MzMzMzMzQ/5mZmZmZmZz/pmZmZmZmaP+zMzMzM
    zM0/8AAAAAAAAD/xmZmZmZmaP/MzMzMzMzQ/9MzMzMzMzT/2ZmZmZmZnP/gAAAAAAAA/+ZmZ
    mZmZmj/7MzMzMzM0P/zMzMzMzM0//mZmZmZmZ0AAAAAAAAAAQADMzMzMzM1AAZmZmZmZmkAC
    ZmZmZmZnQAMzMzMzMzRABAAAAAAAAEAEzMzMzMzNQAWZmZmZmZpABmZmZmZmZ0AHMzMzMzM0
    QAgAAAAAAABACMzMzMzMzUAJmZmZmZmaQApmZmZmZmdACzMzMzMzNEAMAAAAAAAAQAzMzMzM
    zM1ADZmZmZmZmkAOZmZmZmZnQA8zMzMzMzRAEAAAAAAAAAAAAA4AAAApAAAAAAAAAABANAyu
    0zjw50A/lRzdcWYwQEMbtP83YWdARQQoIjRnbUBGHR+kHo/gQEa+veTN9odARxu1QXaZ+UBH
    US8pZOwAQEdv8eQEwVRAR4GjmsQ1UUBHi9ExkBEiQEeRq/wN795AR5UKHxh3/0BHlvoKivxr
    QEeYF04NfgNAR5i7ZQzoUEBHmRnIYYYoQEeZUBOmRjhAR5lvTs7xw0BHmYFFy8yRQEeZi5s7
    ostAR5mRjPH8aEBHmZT4RFiBQEeZlu/FWV1AR5mYEWWvC0BHmZi3/xpsQEeZmRfT9/RAR5mZ
    TvPNMUBHmZluqTtlQEeZmYDmjZNAR5mZi2RydEBHmZmRbW5lQEeZmZTmI7FAR5mZluVX8kBH
    mZmYC2YvQEeZmZi0i9pAR5mZmRXX50BHmZmZTc+OQEeZmZluASBAR5mZmYCF2gAAAA4AAAAp
    f/gAAAAAB6J/+AAAAAAHon/4AAAAAAeif/gAAAAAB6J/+AAAAAAHon/4AAAAAAeif/gAAAAA
    B6J/+AAAAAAHon/4AAAAAAeif/gAAAAAB6J/+AAAAAAHon/4AAAAAAeif/gAAAAAB6J/+AAA
    AAAHon/4AAAAAAeif/gAAAAAB6J/+AAAAAAHon/4AAAAAAeif/gAAAAAB6J/+AAAAAAHon/4
    AAAAAAeif/gAAAAAB6J/+AAAAAAHon/4AAAAAAeif/gAAAAAB6J/+AAAAAAHon/4AAAAAAei
    f/gAAAAAB6J/+AAAAAAHon/4AAAAAAeif/gAAAAAB6J/+AAAAAAHon/4AAAAAAeif/gAAAAA
    B6J/+AAAAAAHon/4AAAAAAeif/gAAAAAB6J/+AAAAAAHon/4AAAAAAeif/gAAAAAB6J/+AAA
    AAAHogAAAA4AAAApAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAD8k
    nXtAp5AiP4N49AmChas/nXENXbNzKT+rfK7vUcovP7T28LyVXfc/vKrvh8s9Uj/CUvSmW12q
    P8Zk27PrQpE/yoKA9eHynz/OpudzrtrqP9FnmFFiVwM/03za/oEwFz/VksI7dWM6P9epCCFA
    Eiw/2b+EekrrpT/b1iAlfhJqP93szdToxWM/4AHC8K+z7D/hDSHx9n1EP+IYgqoov7o/4yPk
    XtTmeT/kL0aku9v+P+U6qT4s3rU/5kYMB6uZMT/nUW7szopDP+hc0eHX30Y/6Wg04AaUnT/q
    c5fjeBlZP+t++unwVOg/7Ipd8iYyWD/tlcD7XGYgP+6hJAUmDU0/76yHD0SFmD/wW/UMyePf
    P/DhppH/jSc/8WdYFz1IwwAAAA4AAAApP/AAAAAAAAA/8AAAAAAAAD/wAAAAAAAAP/AAAAAA
    AAA/8AAAAAAAAD/v/rYu789BP++yeq53ERU/7xfSJImoUD/uU83aZNYMP+173gyhdSQ/7Jwi
    EYXi5z/ru1Zi6+6XP+rdJ84HcNM/6gOKPAYwpT/pL3s2xQwgP+hhb19jd9o/55mPuyWGnj/m
    19vtYrWJP+YcPUl61S0/5WaRbfbi1j/ktrAnSEUKP+QMbrFAkXE/42ehgaXFlD/iyB1CKZrf
    P+Itt1ZiByE/4ZhGIVYVdT/hB6EnVd2KP+B7oRwZOBo/3+g/zBQVOD/e4fE3WTE7P93kDvvL
    fhE/3O5T4UunNj/cAHzloSInP9saSSuy6JM/2jt56qvnRT/ZY9JdUetRP9iTF7G0ek4/18kQ
    +UaAYD/XBYcZa4AyP9ZIRLx7THA/1ZEWQzw8nQAABAIAAAABAAQACQAAAAVuYW1lcwAAABAA
    AAAFAAQACQAAAAR0aW1lAAQACQAAAAJDaQAEAAkAAAABRAAEAAkAAAABSAAEAAkAAAABUwAA
    BAIAAAABAAQACQAAAAlyb3cubmFtZXMAAAANAAAAAoAAAAD////XAAAEAgAAAAEABAAJAAAA
    DWRlc29sdmVfZGlhZ24AAAITAAAABQAAAA0AAAACAAAAKQAAAAQAAAANAAAAFQAAAAAAAAGR
    AAAJaIAAAACAAAAAgAAAAIAAAACAAAAAgAAAAIAAAACAAAAAgAAAAAAAAACAAAAAgAAAAIAA
    AACAAAAAAAAABIAAAACAAAAAgAAAAAAAABMAAAACAAAA/gAAABAAAAAEAAQACQAAAAR0aW1l
    AAQACQAAAAJDaQAEAAkAAAABRAAEAAkAAAABSAAAABAAAAACAAQACQAAAAdkZVNvbHZlAAQA
    CQAAAAZtYXRyaXgAAAAQAAAAAQAEAAkAAAACcmsAAAQCAAAB/wAAABAAAAAFAAQACQAAAANk
    aW0ABAAJAAAABmlzdGF0ZQAEAAkAAAAIZGltbmFtZXMABAAJAAAABWNsYXNzAAQACQAAAAR0
    eXBlAAAA/gAABAIAAAABAAQACQAAAAVjbGFzcwAAABAAAAACAAQACQAAAA5jdmFzaV9zaW11
    bGF0ZQAEAAkAAAAKZGF0YS5mcmFtZQAABAIAAAABAAQACQAAAAxjdmFzaV9zdGF0dXMAAAAQ
    AAAAAQAEAAkAAAAHc3VjY2VzcwAABAIAAAABAAQACQAAAA5kZXNvbHZlX291dHB1dAAAABAA
    AAAAAAAEAgAAAAEABAAJAAAADWRlc29sdmVfY29uZHMAAAATAAAAAAAAAP4=

---

    WAoAAAACAAQDAAACAwAAAAMTAAAABQAAAA4AAAApAAAAAAAAAAA/uZmZmZmZmj/JmZmZmZma
    P9MzMzMzMzQ/2ZmZmZmZmj/gAAAAAAAAP+MzMzMzMzQ/5mZmZmZmZz/pmZmZmZmaP+zMzMzM
    zM0/8AAAAAAAAD/xmZmZmZmaP/MzMzMzMzQ/9MzMzMzMzT/2ZmZmZmZnP/gAAAAAAAA/+ZmZ
    mZmZmj/7MzMzMzM0P/zMzMzMzM0//mZmZmZmZ0AAAAAAAAAAQADMzMzMzM1AAZmZmZmZmkAC
    ZmZmZmZnQAMzMzMzMzRABAAAAAAAAEAEzMzMzMzNQAWZmZmZmZpABmZmZmZmZ0AHMzMzMzM0
    QAgAAAAAAABACMzMzMzMzUAJmZmZmZmaQApmZmZmZmdACzMzMzMzNEAMAAAAAAAAQAzMzMzM
    zM1ADZmZmZmZmkAOZmZmZmZnQA8zMzMzMzRAEAAAAAAAAAAAAA4AAAApAAAAAAAAAABAQRN6
    3R4OQkBK5gym6Cy7QFBGPfNmfUNAUeZAjfIOU0BS1YzYy+2EQFNfMxZlp6lAU65g4g9RPEBT
    2+yBoM1VQFP2H199ipxAVAUxTqlHcUBUDdx5ZhdEQFQS2PyjvARAVBW3Q7RMKUBUF12jA2P/
    QFQYUJgsCJxAVBjcWVGAbkBUGSy9GqdnQFQZWvsKXFhAVBl1lHoEI0BUGYThaTxTQFQZja6E
    JcNAVBmSvo0DcUBUGZWoDtHhQFQZl1TjvgFAVBmYS5AYVEBUGZjZdGODQFQZmSsS58pAVBmZ
    WgXhvkBUGZl1B3TCQFQZmYSQSwVAVBmZjX/a+EBUGZmSo7XkQFQZmZWYnmdAVBmZl0wCOkBU
    GZmYRnRRQFQZmZjWhCBAVBmZmSliLkBUGZmZWQz4QFQZmZl0eEVAVBmZmYQ97QAAAA4AAAAp
    f/gAAAAAB6J/+AAAAAAHon/4AAAAAAeif/gAAAAAB6J/+AAAAAAHon/4AAAAAAeif/gAAAAA
    B6J/+AAAAAAHon/4AAAAAAeif/gAAAAAB6J/+AAAAAAHon/4AAAAAAeif/gAAAAAB6J/+AAA
    AAAHon/4AAAAAAeif/gAAAAAB6J/+AAAAAAHon/4AAAAAAeif/gAAAAAB6J/+AAAAAAHon/4
    AAAAAAeif/gAAAAAB6J/+AAAAAAHon/4AAAAAAeif/gAAAAAB6J/+AAAAAAHon/4AAAAAAei
    f/gAAAAAB6J/+AAAAAAHon/4AAAAAAeif/gAAAAAB6J/+AAAAAAHon/4AAAAAAeif/gAAAAA
    B6J/+AAAAAAHon/4AAAAAAeif/gAAAAAB6J/+AAAAAAHon/4AAAAAAeif/gAAAAAB6J/+AAA
    AAAHogAAAA4AAAApAAAAAAAAAAAAAAAAAAAAAD+egGkp1KTmP8ioQ1MVByo/3GrVJ7792j/n
    4flHDOuHP/E/Qj6RKwM/9tKNpx9U6j/8jY0c2ARPQAEvsZO1z5JABB8uIzKSP0AHEnH0uRE8
    QAoH4il7KSJADP6Sag4p6EAP9frDk3+XQBF25oFXaqdAEvLuFedo60AUbwcveV8CQBXrKlzq
    2D9AF2dTVlv2jkAY43+lbdjDQBpfrd+GL8BAG9vdNBFJ2kAdWA0rFMxzQB7UPX+NKJJAICg3
    BOPNwkAg5k9ZdydJQCGkZ7bvTVZAImKAGYUdUkAjIJh/DEYcQCPesOZExkZAJJzJTnaK30Al
    WuG3N7HAQCYY+iBLUuFAJtcSiY5lX0AnlSry7MIjQChTQ1xa0Y9AKRFbxdHom0Apz3QvTjFY
    QCqNjJjNdvJAK0ulAk50hAAAAA4AAAApP/AAAAAAAAA/8AAAAAAAAD/vD5Xkf551P+pkn0f8
    g8U/5IbG2ZlTHT/eV6wlzQkeP9XHYlUhn7A/zr3rVt02CT/FfM19SPVGP73e/J/6Aqw/tLI9
    tm+erz+soEbQOmoMP6PGr/ppH3o/m07GIKVf9D+S2OJmUu14P4oCuKcKeUs/gfI/J27Vsj94
    w0NG/4rqP3EVX3aZ9Yc/Z5IhQSQuCT9gQriE5rTVP1ZvXnq7ZaU/TvQuHdtR0D9FWoUDnRvM
    Pz12Lff9Plk/NFL74kQlbj8sCpG7Y3i8PyNYJKm0YsA/GrB7G+Pb1T8SaWSGKUy8PwlnE1BD
    +p8/AYYm1RvUVT74LYznwqhlPvCt3bfxJQg+5wMkB9D/Sz7fwAQNfMyvPtXnHi2Dp9Q+zjgm
    JxojOj7E2MnL1G/6PrzDLMLIJ6g+s9d94pJXAwAABAIAAAABAAQACQAAAAVuYW1lcwAAABAA
    AAAFAAQACQAAAAR0aW1lAAQACQAAAAJDaQAEAAkAAAABRAAEAAkAAAABSAAEAAkAAAABUwAA
    BAIAAAABAAQACQAAAAlyb3cubmFtZXMAAAANAAAAAoAAAAD////XAAAEAgAAAAEABAAJAAAA
    DWRlc29sdmVfZGlhZ24AAAITAAAABQAAAA0AAAACAAAAKQAAAAQAAAANAAAAFQAAAAAAAAGR
    AAAJaIAAAACAAAAAgAAAAIAAAACAAAAAgAAAAIAAAACAAAAAgAAAAAAAAACAAAAAgAAAAIAA
    AACAAAAAAAAABIAAAACAAAAAgAAAAAAAABMAAAACAAAA/gAAABAAAAAEAAQACQAAAAR0aW1l
    AAQACQAAAAJDaQAEAAkAAAABRAAEAAkAAAABSAAAABAAAAACAAQACQAAAAdkZVNvbHZlAAQA
    CQAAAAZtYXRyaXgAAAAQAAAAAQAEAAkAAAACcmsAAAQCAAAB/wAAABAAAAAFAAQACQAAAANk
    aW0ABAAJAAAABmlzdGF0ZQAEAAkAAAAIZGltbmFtZXMABAAJAAAABWNsYXNzAAQACQAAAAR0
    eXBlAAAA/gAABAIAAAABAAQACQAAAAVjbGFzcwAAABAAAAACAAQACQAAAA5jdmFzaV9zaW11
    bGF0ZQAEAAkAAAAKZGF0YS5mcmFtZQAABAIAAAABAAQACQAAAAxjdmFzaV9zdGF0dXMAAAAQ
    AAAAAQAEAAkAAAAHc3VjY2VzcwAABAIAAAABAAQACQAAAA5kZXNvbHZlX291dHB1dAAAABAA
    AAAAAAAEAgAAAAEABAAJAAAADWRlc29sdmVfY29uZHMAAAATAAAAAAAAAP4=

