% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/log.R
\name{log_envir}
\alias{log_envir}
\title{Log R environment properties}
\usage{
log_envir()
}
\value{
no return value
}
\description{
Log R environment properties
}
